% Example script for recursive Levenberg-Marquardt reassignment and recursive
% Synchrosqueezing
% 
% select the test signal setting the variable signal in 1-6
%
%
% Author: D. Fourer (dominique@fourer.fr)
% Date: Sept 2015
% Ref: [D.Fourer, F. Auger and P.Flandrin. Recursive versions of the Levenberg-Marquardt
% reassigned spectrogram and of the synchrosqueezed STFT. IEEE Proc. ICASSP 2016.]

clear all
close all

signal=1;
%% used for zero-padding for Reconstruction Quality computation
N_0pad = 100;
z      = zeros(N_0pad, 1);

%% 1: Load test signal
M  = 300;
mi = 1; mf = round(M/2);

k_range   = 5:9;
L_range   = [7];
SNR_range = [45 40 35 30 25 20 15 10 5 0];

%n0_range0 = [8 18 27];
%mu_range = [0.3 0.8 1.3 1.8 2.3 2.8];
%M_range = [100 200 600 1000 2400];

alpha  = 0.4;           %% used for display
%alpha2 = 0.25;          %% used for display

load_signal;

figure(6); 
subplot(211); plot(real(s));
subplot(212); plot(imag(s));

n_range0 = N_0pad+(1:N);

s_ref = s;
Mr = mf-mi+1;

chemin  = sprintf('figs/%d', signal);
chemin0 = sprintf('%s/spec', chemin);     %% spectrogram

if ~exist(chemin0, 'dir')
 mkdir(chemin0);
end

fp = fopen(sprintf('%s/results.txt', chemin0), 'wt');

t = (1:length(s))-1;

index  = 1;                 %% used for figures indices
for rsb_target = SNR_range
  
  s = sigmerge(s_ref,randn(size(s_ref)),rsb_target);
  s = [z;s];                         %% 0 padding used for reconstruction
  rsb = SNR(s_ref, s(n_range0));
  fprintf('rsb_target= %f dB, rsb = %f dB\n', rsb_target, rsb);
  
  if abs(rsb - rsb_target) > 1
    error('Invalid SNR')
  end
    
  rsb = rsb_target;

 for k = k_range
  for L = L_range
    L=28/(k-1);
    fprintf(1,  'Parameters: k=%d L=%0.02f snr=%0.02f M=%d, ', k, L, rsb, M);
    fprintf(fp, 'Parameters: k=%d L=%0.02f snr=%0.02f M=%d,', k, L, rsb, M);
  
    %% 1a) Recursive STFT - index
    [tfr, nfreqs] = recursive_stft(s, k, L, mi, mf, M);
    figure(1)
    imagesc(t, nfreqs, (abs(tfr(:,n_range0)).^2).^alpha);
    set(gca,'YDir','normal')
    colormap gray;
    cmap = colormap;
    cmap = flipud(cmap);
    colormap(cmap); 
    xlabel('time samples', 'FontSize', 16)  %, 'FontName', 'Times-Roman', 'FontSize', 20
    ylabel('normalized frequency', 'FontSize', 16)
    title(sprintf('Recursive spectrogram k=%d, L=%2.2f, SNR=%d dB', k, L, rsb), 'FontSize', 14);
    saveas(gcf, sprintf('%s/rspectrum_%d.eps', chemin0, index), 'epsc');
    index = index + 1;
	
    %% 1b) Recursive STFT Reconstruction
    n0 = round((k-1) * L);
    [ s_hat] = stft_rec(tfr, k, L, mi:mf, M, n0);
    n_range_rec = n_range0-n0;
    s_rec = 2*real(s_hat(n_range_rec)); %% assume the signal is real
    fprintf(1,  'Recursive STFT reconstruction : RQF=%0.02f dB\n', SNR(s(n_range_rec), s_rec));
    fprintf(fp, 'Recursive STFT reconstruction : RQF=%0.02f dB\n', SNR(s(n_range_rec), s_rec));
  end %% L
 end %% k
end % RSB
 
fclose(fp);
eps2pdf(chemin0)